/*
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

#include <lib-lib/lib-lib.h>

#include "mutt.h"
#include "imap_private.h"

#include "mx.h"
#include "mx_imap.h"

int imap_is_magic (const char* path, struct stat* st __attribute__ ((unused))) {
  url_scheme_t s;
  if (!path || !*path)
    return (-1);
  s = url_check_scheme (NONULL (path));
  return ((s == U_IMAP || s == U_IMAPS) ? M_IMAP : -1);
}

static int acl_check_imap (CONTEXT* ctx, int bit) {
  return (!mutt_bit_isset (((IMAP_DATA*) ctx->data)->capabilities, ACL) ||
          mutt_bit_isset (((IMAP_DATA*) ctx->data)->rights, bit));
}

static int imap_open_new_message (MESSAGE * msg,
                                  CONTEXT * dest __attribute__ ((unused)),
                                  HEADER * hdr __attribute__ ((unused)))
{
  char tmp[_POSIX_PATH_MAX];

  mutt_mktemp (tmp);
  if ((msg->fp = safe_fopen (tmp, "w")) == NULL) {
    mutt_perror (tmp);
    return (-1);
  }
  msg->path = m_strdup(tmp);
  return 0;
}

/* this ugly kludge is required since the last int to
 * imap_check_mailbox() doesn't mean 'lock' but 'force'... */
static int _imap_check_mailbox (CONTEXT* ctx,
                                int* index_hint,
                                int lock __attribute__ ((unused))) {
  return (imap_check_mailbox (ctx, index_hint, 0));
}

static int imap_commit_message (MESSAGE* msg, CONTEXT* ctx) {
  int r = 0;

  if ((r = safe_fclose (&msg->fp)) == 0)
    r = imap_append_message (ctx, msg);
  return (r);
}

mx_t const imap_mx = {
    M_IMAP,
    0,
    imap_is_magic,
    NULL,
    imap_access,
    imap_open_mailbox,
    imap_open_new_message,
    acl_check_imap,
    _imap_check_mailbox,
    imap_close_mailbox,
    imap_sync_mailbox,
    imap_commit_message,
};
